# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, division


import sqlite3
from tempfile import NamedTemporaryFile
from petl.compat import next


import petl.io.db as db
from petl.test.helpers import ieq, eq_
from petl.io.db import fromdb, todb, appenddb
from petl.io.db_create import drop_table


# N.B., this file only tests the DB-related functions using sqlite3,
# as anything else requires database connection configuration. See
# docs/dbtests.py for a script to exercise the DB-related functions with
# MySQL and Postgres.


class LoggingConnection(object):

    def __init__(self):
        self.sql = []

    def cursor(self):
        return self

    def execute(self, sql):
        self.sql.append(sql)

    def close(self):
        pass

    def commit(self):
        pass


def test_drop_table_if_exists_sql():
    dbo = LoggingConnection()

    drop_table(dbo, 'test_create', commit=False)
    drop_table(dbo, 'test_create', commit=False, if_exists=True)

    eq_([u'DROP TABLE "test_create"',
         u'DROP TABLE IF EXISTS "test_create"'],
        dbo.sql)


def _record_todb_create_calls(monkeypatch, drop):
    calls = []

    def fake_drop_table(dbo, tablename, schema=None, commit=True,
                        if_exists=False):
        calls.append(('drop', tablename, if_exists))

    def fake_create_table(table, dbo, tablename, schema=None, commit=True,
                          constraints=True, metadata=None, dialect=None,
                          sample=1000):
        calls.append(('create', tablename))

    def fake_todb(table, dbo, tablename, schema=None, commit=True,
                  truncate=False):
        calls.append(('load', tablename, truncate))

    monkeypatch.setattr(db, 'drop_table', fake_drop_table)
    monkeypatch.setattr(db, 'create_table', fake_create_table)
    monkeypatch.setattr(db, '_todb', fake_todb)

    table = (('foo',), ('a',))
    todb(table, object(), 'foobar', create=True, drop=drop)
    return calls


def test_todb_drop_false_skips_drop(monkeypatch):
    calls = _record_todb_create_calls(monkeypatch, False)

    eq_([('create', 'foobar'),
         ('load', 'foobar', True)],
        calls)


def test_todb_drop_true_uses_plain_drop(monkeypatch):
    calls = _record_todb_create_calls(monkeypatch, True)

    eq_([('drop', 'foobar', False),
         ('create', 'foobar'),
         ('load', 'foobar', True)],
        calls)


def test_todb_drop_if_exists_uses_if_exists(monkeypatch):
    calls = _record_todb_create_calls(monkeypatch, 'if_exists')

    eq_([('drop', 'foobar', True),
         ('create', 'foobar'),
         ('load', 'foobar', True)],
        calls)


def test_fromdb():

    # initial data
    data = (('a', 1),
            ('b', 2),
            ('c', 2.0))
    connection = sqlite3.connect(':memory:')
    c = connection.cursor()
    c.execute('create table foobar (foo, bar)')
    for row in data:
        c.execute('insert into foobar values (?, ?)', row)
    connection.commit()
    c.close()

    # test the function
    actual = fromdb(connection, 'select * from foobar', ownsdb=True)
    expect = (('foo', 'bar'),
              ('a', 1),
              ('b', 2),
              ('c', 2.0))
    ieq(expect, actual)
    ieq(expect, actual)  # verify can iterate twice

    # test iterators are isolated
    i1 = iter(actual)
    i2 = iter(actual)
    eq_(('foo', 'bar'), next(i1))
    eq_(('a', 1), next(i1))
    eq_(('foo', 'bar'), next(i2))
    eq_(('b', 2), next(i1))


def test_fromdb_mkcursor():

    # initial data
    data = (('a', 1),
            ('b', 2),
            ('c', 2.0))
    connection = sqlite3.connect(':memory:')
    c = connection.cursor()
    c.execute('create table foobar (foo, bar)')
    for row in data:
        c.execute('insert into foobar values (?, ?)', row)
    connection.commit()
    c.close()

    # test the function
    mkcursor = lambda: connection.cursor()
    actual = fromdb(mkcursor, 'select * from foobar')
    expect = (('foo', 'bar'),
              ('a', 1),
              ('b', 2),
              ('c', 2.0))
    ieq(expect, actual)
    ieq(expect, actual)  # verify can iterate twice

    # test iterators are isolated
    i1 = iter(actual)
    i2 = iter(actual)
    eq_(('foo', 'bar'), next(i1))
    eq_(('a', 1), next(i1))
    eq_(('foo', 'bar'), next(i2))
    eq_(('b', 2), next(i1))


def test_fromdb_withargs():

    # initial data
    data = (('a', 1),
            ('b', 2),
            ('c', 2.0))
    connection = sqlite3.connect(':memory:')
    c = connection.cursor()
    c.execute('create table foobar (foo, bar)')
    for row in data:
        c.execute('insert into foobar values (?, ?)', row)
    connection.commit()
    c.close()

    # test the function
    actual = fromdb(
        connection,
        'select * from foobar where bar > ? and bar < ?',
        (1, 3)
    )
    expect = (('foo', 'bar'),
              ('b', 2),
              ('c', 2.0))
    ieq(expect, actual)
    ieq(expect, actual) # verify can iterate twice
    connection.close()


def test_todb_appenddb():

    f = NamedTemporaryFile(delete=False)
    conn = sqlite3.connect(f.name)
    conn.execute('create table foobar (foo, bar)')
    conn.commit()

    # exercise function
    table = (('foo', 'bar'),
             ('a', 1),
             ('b', 2),
             ('c', 2))
    todb(table, conn, 'foobar')

    # check what it did
    actual = conn.execute('select * from foobar')
    expect = (('a', 1),
              ('b', 2),
              ('c', 2))
    ieq(expect, actual)

    # try appending
    table2 = (('foo', 'bar'),
              ('d', 7),
              ('e', 9),
              ('f', 1))
    appenddb(table2, conn, 'foobar')

    # check what it did
    actual = conn.execute('select * from foobar')
    expect = (('a', 1),
              ('b', 2),
              ('c', 2),
              ('d', 7),
              ('e', 9),
              ('f', 1))
    ieq(expect, actual)
    conn.close()


def test_todb_appenddb_cursor():

    f = NamedTemporaryFile(delete=False)
    conn = sqlite3.connect(f.name)
    conn.execute('create table foobar (foo, bar)')
    conn.commit()

    # exercise function
    table = (('foo', 'bar'),
             ('a', 1),
             ('b', 2),
             ('c', 2))
    cursor = conn.cursor()
    todb(table, cursor, 'foobar')

    # check what it did
    actual = conn.execute('select * from foobar')
    expect = (('a', 1),
              ('b', 2),
              ('c', 2))
    ieq(expect, actual)

    # try appending
    table2 = (('foo', 'bar'),
              ('d', 7),
              ('e', 9),
              ('f', 1))
    appenddb(table2, cursor, 'foobar')

    # check what it did
    actual = conn.execute('select * from foobar')
    expect = (('a', 1),
              ('b', 2),
              ('c', 2),
              ('d', 7),
              ('e', 9),
              ('f', 1))
    ieq(expect, actual)
    conn.close()



# -*- coding: utf-8 -*-
from __future__ import division, print_function, absolute_import


import locale


from petl.compat import izip_longest, next, xrange, BytesIO
from petl.util.base import Table
from petl.io.sources import read_source_from_arg, write_source_from_arg


def fromxls(filename, sheet=None, use_view=True, **kwargs):
    """
    Extract a table from a sheet in an Excel .xls file.
    
    Sheet is identified by its name or index number.
    
    N.B., the sheet name is case sensitive.

    """
    
    return XLSView(filename, sheet=sheet, use_view=use_view, **kwargs)


class XLSView(Table):

    def __init__(self, filename, sheet=None, use_view=True, **kwargs):
        self.filename = filename
        self.sheet = sheet
        self.use_view = use_view
        self.kwargs = kwargs

    def __iter__(self):

        # prefer implementation using xlutils.view as dates are automatically
        # converted
        if self.use_view:
            from petl.io import xlutils_view
            source = read_source_from_arg(self.filename)
            with source.open('rb') as source2:
                source3 = source2.read()
                wb = xlutils_view.View(source3, **self.kwargs)
                if self.sheet is None:
                    ws = wb[0]
                else:
                    ws = wb[self.sheet]
                for row in ws:
                    yield tuple(row)
        else:
            import xlrd
            source = read_source_from_arg(self.filename)
            with source.open('rb') as source2:
                source3 = source2.read()
                with xlrd.open_workbook(file_contents=source3,
                                        on_demand=True, **self.kwargs) as wb:
                    if self.sheet is None:
                        ws = wb.sheet_by_index(0)
                    elif isinstance(self.sheet, int):
                        ws = wb.sheet_by_index(self.sheet)
                    else:
                        ws = wb.sheet_by_name(str(self.sheet))
                    for rownum in xrange(ws.nrows):
                        yield tuple(ws.row_values(rownum))


def toxls(tbl, filename, sheet, encoding=None, style_compression=0,
          styles=None):
    """
    Write a table to a new Excel .xls file.

    """

    import xlwt
    if encoding is None:
        encoding = locale.getpreferredencoding()
    wb = xlwt.Workbook(encoding=encoding, style_compression=style_compression)
    ws = wb.add_sheet(sheet)

    if styles is None:
        # simple version, don't worry about styles
        for r, row in enumerate(tbl):
            for c, v in enumerate(row):
                ws.write(r, c, label=v)
    else:
        # handle styles
        it = iter(tbl)
        try:
            hdr = next(it)
            flds = list(map(str, hdr))
            for c, f in enumerate(flds):
                ws.write(0, c, label=f)
                if f not in styles or styles[f] is None:
                    styles[f] = xlwt.Style.default_style
        except StopIteration:
            pass  # no header written
        # convert to list for easy zipping
        styles = [styles[f] for f in flds]
        for r, row in enumerate(it):
            for c, (v, style) in enumerate(izip_longest(row, styles,
                                                        fillvalue=None)):
                ws.write(r+1, c, label=v, style=style)

    target = write_source_from_arg(filename)
    with target.open('wb') as target2:
        wb.save(target2)


Table.toxls = toxls



import pytest

from petl.io.db_create import make_sqlalchemy_column

try:
    import sqlalchemy  # noqa: F401
except ImportError as e:
    pytest.skip('SKIP generic DB inference tests: %s' % e, allow_module_level=True)
else:
    from sqlalchemy import JSON, Integer

def test_json_inference():
    data = [{'a': 1}, {'b': 2}, None]
    col = make_sqlalchemy_column(data, 'payload')
    assert col.name == 'payload'
    assert isinstance(col.type, JSON)

def test_int_inference():
    col = make_sqlalchemy_column([1, 2, 3], 'n')
    assert col.name == 'n'
    assert isinstance(col.type, Integer)



import random as pyrandom
import time
from functools import partial

from petl.util.random import randomseed, randomtable, RandomTable, dummytable, DummyTable


def test_randomseed():
    """
    Ensure that randomseed provides a non-empty string that changes.
    """
    seed_1 = randomseed()
    time.sleep(1)
    seed_2 = randomseed()

    assert isinstance(seed_1, str)
    assert seed_1 != ""
    assert seed_1 != seed_2


def test_randomtable():
    """
    Ensure that randomtable provides a table with the right number of rows and columns.
    """
    columns, rows = 3, 10
    table = randomtable(columns, rows)

    assert len(table[0]) == columns
    assert len(table) == rows + 1


def test_randomtable_class():
    """
    Ensure that RandomTable provides a table with the right number of rows and columns.
    """
    columns, rows = 4, 60
    table = RandomTable(numflds=columns, numrows=rows)

    assert len(table[0]) == columns
    assert len(table) == rows + 1


def test_dummytable_custom_fields():
    """
    Ensure that dummytable provides a table with the right number of rows
    and that it accepts and uses custom column names provided.
    """
    columns = (
        ('count', partial(pyrandom.randint, 0, 100)),
        ('pet', partial(pyrandom.choice, ['dog', 'cat', 'cow', ])),
        ('color', partial(pyrandom.choice, ['yellow', 'orange', 'brown'])),
        ('value', pyrandom.random),
    )
    rows = 35

    table = dummytable(numrows=rows, fields=columns)
    assert table[0] == ('count', 'pet', 'color', 'value')
    assert len(table) == rows + 1


def test_dummytable_no_seed():
    """
    Ensure that dummytable provides a table with the right number of rows
    and columns when not provided with a seed.
    """
    rows = 35

    table = dummytable(numrows=rows)
    assert len(table[0]) == 3
    assert len(table) == rows + 1


def test_dummytable_int_seed():
    """
    Ensure that dummytable provides a table with the right number of rows
    and columns when provided with an integer as a seed.
    """
    rows = 35
    seed = 42
    table = dummytable(numrows=rows, seed=seed)
    assert len(table[0]) == 3
    assert len(table) == rows + 1


def test_dummytable_class():
    """
    Ensure that DummyTable provides a table with the right number of rows
    and columns.
    """
    rows = 70
    table = DummyTable(numrows=rows)

    assert len(table) == rows + 1
